 /********************************************************************/
 /* MGET     written by Matt Lightfoot, October 1995                 */
 /*                                                                  */
 /* MQSeries Test Program.                                           */
 /*                                                                  */
 /* Statement:    Licensed Materials - Property of IBM               */
 /*                                                                  */
 /*               MA1C SupportPac                                    */
 /*               (c) Copyright IBM Corp. 1995, 1996.                */
 /*                                                                  */
 /*               All rights reserved.                               */
 /*                                                                  */
 /*               U.S. Government Users Restricted Rights - use,     */
 /*               duplication or disclosure restricted by GSA        */
 /*               ADP Schedule Contract with IBM Corp.               */
 /********************************************************************/
 /*                                                                  */
 /* Program name: MGET                                               */
 /*                                                                  */
 /* Description : Gets messages from a queue and prints the          */
 /*               messages in character and hex format               */
 /*                                                                  */
 /* Format:                                                          */
 /* MGET <options> qname <qmgr>                                      */
 /*   options: -b      = browse queue                                */
 /*            -c      = convert data                                */
 /*            -d      = assume the message data begins with a MQDLH */
 /*                      and print the MQDLH header.                 */
 /*            -h      = print MQMD header for each msg              */
 /*            -n<xxx> = get xxx messages, default value             */
 /*                      of xxx is 1, default if -n flag             */
 /*                      not specified is 999999 messages            */
 /*            -i<x>   = open options: -iq _INPUT_AS_Q_DEF (default) */
 /*                                    -is _INPUT_SHARED             */
 /*                                    -ie _INPUT_EXCLUSIVE          */
 /*            -q      = quiet mode, i.e. don't print                */
 /*                      messages                                    */
 /*            -t<xxx> = set buffer size to xxx bytes.               */
 /*                      Default of xxx is 1000.                     */
 /*            -u      = translate queue and qmgr to upper case.     */
 /*            -w<xxx> = set wait interval xxx seconds. Default      */
 /*                      of xxx is 5.                                */
 /*                                                                  */
 /*   qname:   A queue name must be specified.                       */
 /*                                                                  */
 /*   qmgr:    If a queue manager name is not specified, a single    */
 /*            blank is substituted (see #define DEFAULT_QMGR below) */
 /*            which causes the MQI to use the installation's        */
 /*            default queue manager.                                */
 /*                                                                  */
 /*   NOTE: qname must appear before qmgr (if specified), but        */
 /*         options can appear anywhere.                             */
 /*                                                                  */
 /*         Options can be concatenated behind a single '-', thus:   */
 /*                                                                  */
 /*           MGET -b -c -h -n5 MY.QUEUE                             */
 /*           MGET -bch -n5 MY.QUEUE                                 */
 /*           MGET -bchn5 MY.QUEUE                                   */
 /*                                                                  */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
     /* includes for MQI  */
 #include <cmqc.h>

 /********************************************************************/
 /* Set up some defaults. These may be changed if required.          */
 /********************************************************************/
 #define DEFAULT_QMGR " "
 #define DEFAULT_WAIT_SECS 5
 #define MAX_MSG_SIZE 1000

 /********************************************************************/
 /* Declare internal functions.                                      */
 /* NOTE: strupr() is a non-standard C function that isn't supplied  */
 /*       with all compilers and so is redefined here.               */
 /********************************************************************/
 char *strupr(char *string);
 int  ParmTest(char *parm);
 void PrintMD(MQMD *md);
 void PrintDLH(MQDLH *dlh);
 void ConvertMQRC(MQLONG MQRC, char *MQRCText);

 /********************************************************************/
 /* Global flags/variables.                                          */
 /********************************************************************/
 long     browse=0,quiet=0,number=999999,header=0,getwait=0;
 long     convert=0,input_asqdef=1,input_shared=0,input_exclusive=0;
 long     bufsize=MAX_MSG_SIZE,uppercase=0,dlheader=0;


 /********************************************************************/
 /* Main function.                                                   */
 /********************************************************************/
 int main(int argc, char **argv)
 {

   /******************************************************************/
   /* Declare MQI structures etc.                                    */
   /******************************************************************/
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */
   MQDLH   dlh;                     /* DLQ Header                    */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   MQBYTE   *pbuffer;
   MQLONG   buflen;                 /* buffer length                 */
   MQLONG   messlen;                /* message length received       */
   char     QMName??(50??),*pbuff;  /* queue manager name            */
   long     i, msgcount=0;
   char     MQRCText??(31??)={"Return Code not recognised"};

   /******************************************************************/
   /* If no parms were supplied, print some usage messages.          */
   /******************************************************************/
   if (argc < 2)
   {
     printf("MGET - Gets messages from a queue and prints the\n");
     printf("      messages in character and hex format.\n");
     printf(" \n");
     printf("MGET <options> qname <qmgr>\n");
     printf("  options: -b      = browse queue\n");
     printf("           -c      = convert data\n");
     printf("           -d      = assume message data begins with\n");
     printf("                     a MQDLH, and print the MQDLH\n");
     printf("           -h      = print MQMD header for each msg\n");
     printf("           -n<xxx> = get xxx messages, default value\n");
     printf("                     of xxx is 1, default if -n flag\n");
     printf("                     not specified is 999999 messages\n");
     printf("           -i<x>   = open options:\n");
     printf("                       -iq _INPUT_AS_Q_DEF (default)\n");
     printf("                       -is _INPUT_SHARED\n");
     printf("                       -ie _INPUT_EXCLUSIVE\n");
     printf("           -q      = quiet mode, i.e. don't print\n");
     printf("                      messages\n");
     printf("           -t<xxx> = set buffer size to xxx bytes\n");
     printf("                     Default of xxx is 1000.\n");
     printf("           -u      = translate queue and qmgr to\n");
     printf("                     upper case.\n");
     printf("           -w<xxx> = set wait interval xxx seconds.\n");
     printf("                     Default of xxx is 5.\n");
     printf(" \n");
     exit(99);
   }

   /******************************************************************/
   /* Get queue name, queue manager and options from the input parms.*/
   /******************************************************************/
   od.ObjectName??(0??) = ' ';
   strcpy(QMName,DEFAULT_QMGR);

   for(i=1;i<argc;i++)
   {
      if(!strncmp(argv??(i??),"-",1)) ParmTest(argv??(i??));
      else
      {
         if(od.ObjectName??(0??)==' ')
            if(uppercase)
               strcpy( od.ObjectName,strupr(argv??(i??)) );
            else
               strcpy( od.ObjectName,argv??(i??) );
         else
            if(uppercase)
               strcpy( QMName,strupr(argv??(i??)) );
            else
               strcpy( QMName,argv??(i??) );
      }
   }

   /******************************************************************/
   /* Print title messages to confirm what we're doing.              */
   /******************************************************************/
   printf("About to get ");
   if(browse) printf("(browse) ");
   if(number == 1) printf("%d message ",number);
   else printf("%d messages ",number);
   printf("from:\n");
   printf("  Qname  = %s\n",od.ObjectName);
   printf("  Qmgr   = %s\n",QMName);
   if(convert) printf("Wait interval is %d seconds\n",getwait/1000);
   if(getwait) printf("Wait interval is %d seconds\n",getwait/1000);
   if(quiet) printf("Quiet mode - Messages will not be printed\n");

   if(number < 1)
   {
      printf("Cannot get < 1 messages\n");
      exit(4);
   }

   /******************************************************************/
   /* Get storage for the MQGET message buffer                       */
   /******************************************************************/
   pbuffer=(MQBYTE *)malloc(bufsize);
   if(pbuffer == NULL)
   {
      printf("Could not malloc() storage for get buffer\n");
      exit(16);
   }
   buflen=bufsize;
   printf("Buffer size is %d bytes\n",buflen);

   /******************************************************************/
   /* Connect to queue manager.                                      */
   /******************************************************************/
   MQCONN(QMName,                  /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   /* Report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     ConvertMQRC(CReason, MQRCText);
     printf("MQCONN ended with reason code %ld = %s\n", CReason,
             MQRCText);
     exit(CReason);
   }

   /******************************************************************/
   /* Open the named message queue for input.                        */
   /******************************************************************/
   O_options = MQOO_FAIL_IF_QUIESCING
             + MQOO_BROWSE;
   if(input_asqdef)    O_options += MQOO_INPUT_AS_Q_DEF;
   if(input_shared)    O_options += MQOO_INPUT_SHARED;
   if(input_exclusive) O_options += MQOO_INPUT_EXCLUSIVE;

   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* completion code              */
          &Reason);                  /* reason code                  */

   /* Report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     ConvertMQRC(Reason, MQRCText);
     printf("MQOPEN ended with reason code %ld = %s\n", Reason,
             MQRCText);
   }

   if (OpenCode == MQCC_FAILED)
   {
     printf("Unable to open queue for input\n");
   }

   /******************************************************************/
   /* Set up GMO options.                                            */
   /* NOTE: ACCEPT_TRUNCATED_MSG is specified for all MQGETs.        */
   /******************************************************************/
   CompCode = OpenCode;       /* use MQOPEN result for initial test  */
   gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
   if(browse)
      gmo.Options += MQGMO_BROWSE_FIRST;
   if(convert)
      gmo.Options += MQGMO_CONVERT;
   if(getwait)
      gmo.Options += MQGMO_WAIT;
   gmo.WaitInterval = getwait;

   /******************************************************************/
   /* Get messages from the message queue until there is a failure   */
   /* or the number of msgs required have been retrieved.            */
   /******************************************************************/
   while ((CompCode != MQCC_FAILED) & (number > 0) )
   {
     number--;
     if(quiet) buflen=0;          /* if quiet mode, don't want msg   */

     /****************************************************************/
     /* In order to read the messages in sequence, MsgId and         */
     /* CorrelID must have the default value.  MQGET sets them       */
     /* to the values in the message it returns, so re-initialise    */
     /* them before every call.                                      */
     /****************************************************************/
     memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
     memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));
     md.CodedCharSetId = MQCCSI_Q_MGR;
     md.Encoding = MQENC_NATIVE;

     MQGET(Hcon,                /* connection handle                 */
           Hobj,                /* object handle                     */
           &md,                 /* message descriptor                */
           &gmo,                /* get message options               */
           buflen,              /* buffer length                     */
           pbuffer,             /* message buffer                    */
           &messlen,            /* message length                    */
           &CompCode,           /* completion code                   */
           &Reason);            /* reason code                       */

     if (CompCode == MQCC_FAILED)
     {
       if (Reason == MQRC_NO_MSG_AVAILABLE)
       {                         /* special report for normal end    */
         if(!quiet) printf("No more messages\n");
       }
       else                      /* general report for other reasons */
       {
         ConvertMQRC(Reason, MQRCText);
         if(!quiet) printf("MQGET ended with reason code %ld = %s\n",
                 Reason, MQRCText);
       }
     }

     /****************************************************************/
     /* Reset GMO options. NOTE: BROWSE_NEXT now!                    */
     /****************************************************************/
     gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
     if(browse)
        gmo.Options += MQGMO_BROWSE_NEXT;
     if(convert)
        gmo.Options += MQGMO_CONVERT;
     if(getwait)
        gmo.Options += MQGMO_WAIT;
     gmo.WaitInterval = getwait;

     /****************************************************************/
     /* Display each message received.                               */
     /****************************************************************/
     if(CompCode != MQCC_FAILED) msgcount++;
     if ((CompCode != MQCC_FAILED) & (!quiet))
     {
       printf(" \n");
       printf("**** MESSAGE %d ****\n",msgcount);
       ConvertMQRC(Reason, MQRCText);
       printf("CompCode = %d, Reason = %d %s\n",CompCode,Reason,
               MQRCText);
       printf(" \n");
       /**************************************************************/
       /* Print the Message Descriptor, if requested.                */
       /**************************************************************/
       if(header)
       {
          printf("Message Descriptor\n");
          printf("------------------\n");
          PrintMD(&md);
          printf(" \n");
       }
       pbuff = (char *)pbuffer;
       printf("Message Length = %d , ",messlen);
       if(messlen > buflen) messlen = buflen;
       printf("Received Length = %d\n",messlen);
       printf(" \n");
       /**************************************************************/
       /* Print the Dead Letter Header, if requested and if there is */
       /* a complete DLH.                                            */
       /**************************************************************/
       if( dlheader &&
           !strncmp(pbuff,MQDLH_STRUC_ID,sizeof(dlh.StrucId)) &&
           messlen >= sizeof(dlh) )
       {
          printf("Dead Letter Header\n");
          printf("------------------\n");
          PrintDLH((MQDLH *)pbuff);
          printf(" \n");
          /***********************************************************/
          /* Adjust buffer pointer and message length so that the    */
          /* DLH doesn't appear in the hex/char display.             */
          /***********************************************************/
          pbuff = (char *)pbuffer + sizeof(dlh);
          messlen -= sizeof(dlh);
       }
       /**************************************************************/
       /* Print the Message data, taking 16 bytes at a time.         */
       /**************************************************************/
       printf("Message Data\n");
       printf("------------\n");
       for(i=0;i<messlen;i+=16,pbuff+=16)
       {
          int j;
          long bytesleft;
          char printchar;
          if((bytesleft=messlen-i) >= 16) bytesleft=16;

          /***********************************************************/
          /* First print a row of hex chars.                         */
          /***********************************************************/
          printf("%8.8X :",i);
          for(j=0;j<bytesleft;j++) {
             if(j % 4 == 0) printf(" %2.2X",*(pbuff+j));
             else printf("%2.2X",*(pbuff+j));
          }
          /***********************************************************/
          /* This pads the hex display with blanks if this is the    */
          /* last row and it's less than 16 bytes long.              */
          /***********************************************************/
          for(j=bytesleft;j<16;j++) {
             if(j % 4 == 0) printf("   ");
             else printf("  ");
          }

          /***********************************************************/
          /* Next print the characters themselves (if they're        */
          /* printable).                                             */
          /***********************************************************/
          printf(" <");
          for(j=0;j<bytesleft;j++) {
             printchar = isprint(*(pbuff+j)) ? *(pbuff+j) : '.';
             printf("%c",printchar);
          }
          /***********************************************************/
          /* ..and pad the last row with blanks if necessary.        */
          /***********************************************************/
          for(j=bytesleft;j<16;j++) printf(" ");
          printf(">\n");
       }
       printf(" \n");
     }
   }
   printf("TOTAL MESSAGES READ = %d\n",msgcount);
   printf(" \n");

   /******************************************************************/
   /* Free buffer storage.                                           */
   /******************************************************************/
   free(pbuffer);

   /******************************************************************/
   /* Close the queue (if it was opened).                            */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                   /* no close options            */
     MQCLOSE(Hcon,                    /* connection handle           */
             &Hobj,                   /* object handle               */
             C_options,
             &CompCode,               /* completion code             */
             &Reason);                /* reason code                 */

     /* Report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       ConvertMQRC(Reason, MQRCText);
       printf("MQCLOSE ended with reason code %ld = %s\n", Reason,
               MQRCText);
     }
   }

   /******************************************************************/
   /* Disconnect from MQM.                                           */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED )
   {
     MQDISC(&Hcon,                     /* connection handle          */
            &CompCode,                 /* completion code            */
            &Reason);                  /* reason code                */

     /* Report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       ConvertMQRC(Reason, MQRCText);
       printf("MQDISC ended with reason code %ld = %s\n", Reason,
               MQRCText);
     }
   }

   return(0);
 } /* End main() */

 /********************************************************************/
 /* Function to convert a string to upper case.                      */
 /********************************************************************/
 char *strupr(char *string)
 {
    int i, string_len;
    char *in_string;

    string_len=strlen(string);
    in_string=string;

    for(i=0;i<string_len;i++,string++)
    {
       *string=toupper(*string);
    } /* End for */

    return(in_string);

 } /* End strupr() */

 /********************************************************************/
 /* Function to extract option settings.                             */
 /********************************************************************/
 int  ParmTest(char *parm)
 {
    char *numend;
    int  i;

    /*****************************************************************/
    /* Skip the '-' that precedes the options.                       */
    /*****************************************************************/
    parm++;

    /*****************************************************************/
    /* Examine each option until end-of-string.                      */
    /*****************************************************************/
    while(*parm != '\0')
    {
      switch(toupper(*parm))
       {
       case 'B':
          browse=1;
          break;
       case 'C':
          convert=1;
          break;
       case 'D':
          dlheader=1;
          break;
       case 'H':
          header=1;
          break;
       case 'Q':
          quiet=1;
          break;
       case 'N':
          /***********************************************************/
          /* If a number follows 'N', strtol() will interpret it.    */
          /***********************************************************/
          number=strtol(parm+1,&numend,10);
          if(number<=0) number=1;
          break;
       case 'I':
          switch(toupper(*(parm+1)))
          {
           case 'Q':
              input_asqdef=1;
              input_shared=0;
              input_exclusive=0;
              parm++;
              break;
           case 'S':
              input_asqdef=0;
              input_shared=1;
              input_exclusive=0;
              parm++;
              break;
           case 'E':
              input_asqdef=0;
              input_shared=0;
              input_exclusive=1;
              parm++;
              break;
          } /* End switch */
          break;
       case 'T':
          bufsize=strtol(parm+1,&numend,10);
          if(bufsize<=0) bufsize=1000;
          break;
       case 'W':
          /***********************************************************/
          /* Convert wait seconds to milliseconds.                   */
          /***********************************************************/
          getwait=1000 * strtol(parm+1,&numend,10);
          if(getwait<=0) getwait=DEFAULT_WAIT_SECS * 1000;
          break;
       case 'U':
          uppercase=1;
          break;
       } /* End switch */

       parm++;

    } /* End while */

    return(0);

 } /* End ParmTest() */

 /********************************************************************/
 /* Function to print the Message Descriptor.                        */
 /********************************************************************/
 void PrintMD(MQMD *md)
 {
    int i;
    char StrucId??(5??), Format??(9??), ReplyToQ??(49??);
    char ReplyToQMgr??(49??);
    char UserIdentifier??(13??), ApplIdentityData??(33??);
    char PutApplName??(29??);
    char MsgId??(25??), CorrelId??(25??), AccountingToken??(33??);
    char PutDate??(9??), PutTime??(9??), ApplOriginData??(5??);
    char printchar;

    StrucId??(4??) = Format??(8??) = ReplyToQ??(48??) =
       ReplyToQMgr??(48??) =
       UserIdentifier??(12??) = ApplIdentityData??(32??) =
       PutApplName??(28??) =
       MsgId??(24??) = CorrelId??(24??) = AccountingToken??(32??) =
       PutDate??(8??) = PutTime??(8??) = ApplOriginData??(4??) = '\0';

    strncpy(StrucId,md->StrucId,4);
    printf("StrucId         :%s\t\t",StrucId);
    printf("Version         :%d\n",md->Version);
    printf("Report          :%8.8X\t",md->Report);
    printf("MsgType         :%6.8X\n",md->MsgType);
    printf("Expiry          :%8.8X\t",md->Expiry);
    printf("Feedback        :%8.8X\n",md->Feedback);
    printf("Encoding        :%8.8X\t",md->Encoding);
    printf("CodedCharSetId  :%8.8X (%d)\n",md->CodedCharSetId,
                                           md->CodedCharSetId);
    strncpy(Format,md->Format,8);
    printf("Format          :%s\n",Format);
    printf("Priority        :%8.8X\t",md->Priority);
    printf("Persistence     :%8.8X\n",md->Persistence);
    printf("MsgId           :");
    /*****************************************************************/
    /* MsgId is not null terminated, so print one char at a time (if */
    /* the char is printable). Similarly for other strings below.    */
    /*****************************************************************/
    for(i=0;i<24;i++) printf("%c",(isprint(md->MsgId??(i??)) \
                                   ? md->MsgId??(i??) : '.') );
    printf("\n");
    printf("                <");
    for(i=0;i<24;i++) printf("%1.1X",(md->MsgId??(i??) >> 4) );
    printf(">\n");
    printf("                <");
    for(i=0;i<24;i++) printf("%1.1X",(md->MsgId??(i??) & 0x0f) );
    printf(">\n");
    printf("CorrelId        :");
    for(i=0;i<24;i++)
    for(i=0;i<24;i++) printf("%c",(isprint(md->CorrelId??(i??)) \
                                   ? md->CorrelId??(i??) : '.') );
    printf("\n");
    printf("                <");
    for(i=0;i<24;i++) printf("%1.1X",(md->CorrelId??(i??) >> 4) );
    printf(">\n");
    printf("                <");
    for(i=0;i<24;i++) printf("%1.1X",(md->CorrelId??(i??) & 0x0f) );
    printf(">\n");
    printf("BackoutCount    :%8.8X\n",md->BackoutCount);
    strncpy(ReplyToQ,md->ReplyToQ,48);
    printf("ReplyToQ        :%s\n",ReplyToQ);
    strncpy(ReplyToQMgr,md->ReplyToQMgr,48);
    printf("ReplyToQMgr     :%s\n",ReplyToQMgr);
    strncpy(UserIdentifier,md->UserIdentifier,12);
    printf("UserIdentifier  :%s\n",UserIdentifier);
    printf("AccountingToken\n");
    printf("<");
    for(i=0;i<32;i++) printf("%2.2X",md->AccountingToken??(i??));
    printf(">\n");
    strncpy(ApplIdentityData,md->ApplIdentityData,32);
    printf("ApplIdentityData:%s\n",ApplIdentityData);
    printf("PutApplType     :%8.8X\n",md->PutApplType);
    strncpy(PutApplName,md->PutApplName,28);
    printf("PutApplName     :%s\n",PutApplName);
    strncpy(PutDate,md->PutDate,8);
    printf("PutDate         :%s\t",PutDate);
    strncpy(PutTime,md->PutTime,8);
    printf("PutTime         :%s\n",PutTime);
    strncpy(ApplOriginData,md->ApplOriginData,4);
    printf("ApplOriginData  :%s\n",ApplOriginData);
 }  /* End PrintMD() */

 /********************************************************************/
 /* Function to print the Dead Letter Header.                        */
 /********************************************************************/
 void PrintDLH(MQDLH *dlh)
 {
    int i;
    char StrucId??(5??), Format??(9??);
    char DestQName??(49??);
    char DestQMgrName??(49??);
    char PutApplName??(29??);
    char PutDate??(9??), PutTime??(9??);
    char printchar;

    StrucId??(4??) = Format??(8??) =
       DestQName??(48??) =
       DestQMgrName??(48??) =
       PutApplName??(28??) =
       PutDate??(8??) = PutTime??(8??) = '\0';

    strncpy(StrucId,dlh->StrucId,4);
    printf("StrucId         :%s\t\t",StrucId);
    printf("Version         :%d\n",dlh->Version);
    printf("Reason          :%8.8X (%d)\n",dlh->Reason,
                                           dlh->Reason);
    strncpy(DestQName,dlh->DestQName,48);
    printf("DestQName       :%s\n",DestQName);
    strncpy(DestQMgrName,dlh->DestQMgrName,48);
    printf("DestQMgrName    :%s\n",DestQMgrName);
    printf("Encoding        :%8.8X\t",dlh->Encoding);
    printf("CodedCharSetId  :%8.8X (%d)\n",dlh->CodedCharSetId,
                                           dlh->CodedCharSetId);
    strncpy(Format,dlh->Format,8);
    printf("Format          :%s\n",Format);
    printf("PutApplType     :%8.8X\n",dlh->PutApplType);
    strncpy(PutApplName,dlh->PutApplName,28);
    printf("PutApplName     :%s\n",PutApplName);
    strncpy(PutDate,dlh->PutDate,8);
    printf("PutDate         :%s\t",PutDate);
    strncpy(PutTime,dlh->PutTime,8);
    printf("PutTime         :%s\n",PutTime);
 } /* End PrintDLH() */

 /********************************************************************/
 /* Function to convert an MQRC number into sensible text.           */
 /********************************************************************/
 void  ConvertMQRC(MQLONG MQRC, char *MQRCText)
 {

    /*****************************************************************/
    /* If new msgs are added to MQRCTextArray, NO_OF_MQRCS must be   */
    /* increased accordingly.                                        */
    /*****************************************************************/
    #define NO_OF_MQRCS 173

    typedef struct tagMQRCTEXT {
        MQLONG RC;
        char  RCText??(31??);
        } MQRCTEXT;
    int i;
    /*****************************************************************/
    /* Array of MQRC message numbers and message texts.              */
    /*****************************************************************/
    MQRCTEXT MQRCTextArray??(NO_OF_MQRCS??) =
      {
      0,    "MQRC_NONE                    ",
      2001, "MQRC_ALIAS_BASE_Q_TYPE_ERROR ",
      2002, "MQRC_ALREADY_CONNECTED       ",
      2003, "MQRC_BACKED_OUT              ",
      2004, "MQRC_BUFFER_ERROR            ",
      2005, "MQRC_BUFFER_LENGTH_ERROR     ",
      2006, "MQRC_CHAR_ATTR_LENGTH_ERROR  ",
      2007, "MQRC_CHAR_ATTRS_ERROR        ",
      2008, "MQRC_CHAR_ATTRS_TOO_SHORT    ",
      2009, "MQRC_CONNECTION_BROKEN       ",
      2010, "MQRC_DATA_LENGTH_ERROR       ",
      2011, "MQRC_DYNAMIC_Q_NAME_ERROR    ",
      2012, "MQRC_ENVIRONMENT_ERROR       ",
      2013, "MQRC_EXPIRY_ERROR            ",
      2014, "MQRC_FEEDBACK_ERROR          ",
      2016, "MQRC_GET_INHIBITED           ",
      2017, "MQRC_HANDLE_NOT_AVAILABLE    ",
      2018, "MQRC_HCONN_ERROR             ",
      2019, "MQRC_HOBJ_ERROR              ",
      2020, "MQRC_INHIBIT_VALUE_ERROR     ",
      2021, "MQRC_INT_ATTR_COUNT_ERROR    ",
      2022, "MQRC_INT_ATTR_COUNT_TOO_SMALL ",
      2023, "MQRC_INT_ATTRS_ARRAY_ERROR   ",
      2024, "MQRC_SYNCPOINT_LIMIT_REACHED ",
      2025, "MQRC_MAX_CONNS_LIMIT_REACHED ",
      2026, "MQRC_MD_ERROR                ",
      2027, "MQRC_MISSING_REPLY_TO_Q      ",
      2029, "MQRC_MSG_TYPE_ERROR          ",
      2030, "MQRC_MSG_TOO_BIG_FOR_Q       ",
      2031, "MQRC_MSG_TOO_BIG_FOR_Q_MGR   ",
      2033, "MQRC_NO_MSG_AVAILABLE        ",
      2034, "MQRC_NO_MSG_UNDER_CURSOR     ",
      2035, "MQRC_NOT_AUTHORIZED          ",
      2036, "MQRC_NOT_OPEN_FOR_BROWSE     ",
      2037, "MQRC_NOT_OPEN_FOR_INPUT      ",
      2038, "MQRC_NOT_OPEN_FOR_INQUIRE    ",
      2039, "MQRC_NOT_OPEN_FOR_OUTPUT     ",
      2040, "MQRC_NOT_OPEN_FOR_SET        ",
      2041, "MQRC_OBJECT_CHANGED          ",
      2042, "MQRC_OBJECT_IN_USE           ",
      2043, "MQRC_OBJECT_TYPE_ERROR       ",
      2044, "MQRC_OD_ERROR                ",
      2045, "MQRC_OPTION_NOT_VALID_FOR_TYPE",
      2046, "MQRC_OPTIONS_ERROR           ",
      2047, "MQRC_PERSISTENCE_ERROR       ",
      2048, "MQRC_PERSISTENT_NOT_ALLOWED  ",
      2049, "MQRC_PRIORITY_EXCEEDS_MAXIMUM ",
      2050, "MQRC_PRIORITY_ERROR          ",
      2051, "MQRC_PUT_INHIBITED           ",
      2052, "MQRC_Q_DELETED               ",
      2053, "MQRC_Q_FULL                  ",
      2055, "MQRC_Q_NOT_EMPTY             ",
      2056, "MQRC_Q_SPACE_NOT_AVAILABLE   ",
      2057, "MQRC_Q_TYPE_ERROR            ",
      2058, "MQRC_Q_MGR_NAME_ERROR        ",
      2059, "MQRC_Q_MGR_NOT_AVAILABLE     ",
      2061, "MQRC_REPORT_OPTIONS_ERROR    ",
      2062, "MQRC_SECOND_MARK_NOT_ALLOWED ",
      2063, "MQRC_SECURITY_ERROR          ",
      2065, "MQRC_SELECTOR_COUNT_ERROR    ",
      2066, "MQRC_SELECTOR_LIMIT_EXCEEDED ",
      2067, "MQRC_SELECTOR_ERROR          ",
      2068, "MQRC_SELECTOR_NOT_FOR_TYPE   ",
      2069, "MQRC_SIGNAL_OUTSTANDING      ",
      2070, "MQRC_SIGNAL_REQUEST_ACCEPTED ",
      2071, "MQRC_STORAGE_NOT_AVAILABLE   ",
      2072, "MQRC_SYNCPOINT_NOT_AVAILABLE ",
      2075, "MQRC_TRIGGER_CONTROL_ERROR   ",
      2076, "MQRC_TRIGGER_DEPTH_ERROR     ",
      2077, "MQRC_TRIGGER_MSG_PRIORITY_ERR ",
      2078, "MQRC_TRIGGER_TYPE_ERROR      ",
      2079, "MQRC_TRUNCATED_MSG_ACCEPTED  ",
      2080, "MQRC_TRUNCATED_MSG_FAILED    ",
      2082, "MQRC_UNKNOWN_ALIAS_BASE_Q    ",
      2085, "MQRC_UNKNOWN_OBJECT_NAME     ",
      2086, "MQRC_UNKNOWN_OBJECT_Q_MGR    ",
      2087, "MQRC_UNKNOWN_REMOTE_Q_MGR    ",
      2090, "MQRC_WAIT_INTERVAL_ERROR     ",
      2091, "MQRC_XMIT_Q_TYPE_ERROR       ",
      2092, "MQRC_XMIT_Q_USAGE_ERROR      ",
      2093, "MQRC_NOT_OPEN_FOR_PASS_ALL   ",
      2094, "MQRC_NOT_OPEN_FOR_PASS_IDENT ",
      2095, "MQRC_NOT_OPEN_FOR_SET_ALL    ",
      2096, "MQRC_NOT_OPEN_FOR_SET_IDENT  ",
      2097, "MQRC_CONTEXT_HANDLE_ERROR    ",
      2098, "MQRC_CONTEXT_NOT_AVAILABLE   ",
      2099, "MQRC_SIGNAL1_ERROR           ",
      2100, "MQRC_OBJECT_ALREADY_EXISTS   ",
      2101, "MQRC_OBJECT_DAMAGED          ",
      2102, "MQRC_RESOURCE_PROBLEM        ",
      2103, "MQRC_ANOTHER_Q_MGR_CONNECTED ",
      2104, "MQRC_UNKNOWN_REPORT_OPTION   ",
      2105, "MQRC_STORAGE_CLASS_ERROR     ",
      2107, "MQRC_XWAIT_CANCELED          ",
      2108, "MQRC_XWAIT_ERROR             ",
      2109, "MQRC_SUPPRESSED_BY_EXIT      ",
      2110, "MQRC_FORMAT_ERROR            ",
      2111, "MQRC_SOURCE_CCSID_ERROR      ",
      2112, "MQRC_SOURCE_INTEGER_ENC_ERROR ",
      2113, "MQRC_SOURCE_DECIMAL_ENC_ERROR ",
      2114, "MQRC_SOURCE_FLOAT_ENC_ERROR  ",
      2115, "MQRC_TARGET_CCSID_ERROR      ",
      2116, "MQRC_TARGET_INTEGER_ENC_ERROR ",
      2117, "MQRC_TARGET_DECIMAL_ENC_ERROR ",
      2118, "MQRC_TARGET_FLOAT_ENC_ERROR  ",
      2119, "MQRC_NOT_CONVERTED           ",
      2120, "MQRC_CONVERTED_MSG_TOO_BIG   ",
      2127, "MQRC_ADAPTER_STORAGE_SHORTAGE ",
      2129, "MQRC_ADAPTER_CONN_LOAD_ERROR ",
      2130, "MQRC_ADAPTER_SERV_LOAD_ERROR ",
      2131, "MQRC_ADAPTER_DEFS_ERROR      ",
      2132, "MQRC_ADAPTER_DEFS_LOAD_ERROR ",
      2138, "MQRC_ADAPTER_DISC_LOAD_ERROR ",
      2140, "MQRC_CICS_WAIT_FAILED        ",
      2143, "MQRC_SOURCE_LENGTH_ERROR     ",
      2144, "MQRC_TARGET_LENGTH_ERROR     ",
      2145, "MQRC_SOURCE_BUFFER_ERROR     ",
      2146, "MQRC_TARGET_BUFFER_ERROR     ",
      2150, "MQRC_DBCS_ERROR              ",
      2151, "MQRC_TRUNCATED               ",
      2157, "MQRC_ASID_MISMATCH           ",
      2160, "MQRC_CONN_ID_IN_USE          ",
      2161, "MQRC_Q_MGR_QUIESCING         ",
      2162, "MQRC_Q_MGR_STOPPING          ",
      2163, "MQRC_DUPLICATE_RECOV_COORD   ",
      2173, "MQRC_PMO_ERROR               ",
      2182, "MQRC_API_EXIT_NOT_FOUND      ",
      2183, "MQRC_API_EXIT_LOAD_ERROR     ",
      2184, "MQRC_REMOTE_Q_NAME_ERROR     ",
      2186, "MQRC_GMO_ERROR               ",
      2192, "MQRC_PAGESET_FULL            ",
      2193, "MQRC_PAGESET_ERROR           ",
      2194, "MQRC_NAME_NOT_VALID_FOR_TYPE ",
      2195, "MQRC_UNEXPECTED_ERROR        ",
      2196, "MQRC_UNKNOWN_XMIT_Q          ",
      2197, "MQRC_UNKNOWN_DEF_XMIT_Q      ",
      2198, "MQRC_DEF_XMIT_Q_TYPE_ERROR   ",
      2199, "MQRC_DEF_XMIT_Q_USAGE_ERROR  ",
      2201, "MQRC_NAME_IN_USE             ",
      2202, "MQRC_CONNECTION_QUIESCING    ",
      2203, "MQRC_CONNECTION_STOPPING     ",
      2204, "MQRC_ADAPTER_NOT_AVAILABLE   ",
      2206, "MQRC_MSG_ID_ERROR            ",
      2207, "MQRC_CORREL_ID_ERROR         ",
      2208, "MQRC_FILE_SYSTEM_ERROR       ",
      2209, "MQRC_NO_MSG_LOCKED           ",
      2216, "MQRC_FILE_NOT_AUDITED        ",
      2217, "MQRC_CONNECTION_NOT_AUTHORIZED",
      2218, "MQRC_MSG_TOO_BIG_FOR_CHANNEL ",
      2219, "MQRC_CALL_IN_PROGRESS        ",
      2222, "MQRC_Q_MGR_ACTIVE            ",
      2223, "MQRC_Q_MGR_NOT_ACTIVE        ",
      2224, "MQRC_Q_DEPTH_HIGH            ",
      2225, "MQRC_Q_DEPTH_LOW             ",
      2226, "MQRC_Q_SERVICE_INTERVAL_HIGH ",
      2227, "MQRC_Q_SERVICE_INTERVAL_OK   ",
      2280, "MQRC_HCONFIG_ERROR           ",
      2281, "MQRC_FUNCTION_ERROR          ",
      2282, "MQRC_CHANNEL_STARTED         ",
      2283, "MQRC_CHANNEL_STOPPED         ",
      2284, "MQRC_CHANNEL_CONV_ERROR      ",
      2285, "MQRC_SERVICE_NOT_AVAILABLE   ",
      2286, "MQRC_INITIALIZATION_FAILED   ",
      2287, "MQRC_TERMINATION_FAILED      ",
      2288, "MQRC_UNKNOWN_Q_NAME          ",
      2289, "MQRC_SERVICE_ERROR           ",
      2290, "MQRC_Q_ALREADY_EXISTS        ",
      2291, "MQRC_USER_ID_NOT_AVAILABLE   ",
      2292, "MQRC_UNKNOWN_ENTITY          ",
      2293, "MQRC_UNKNOWN_AUTH_ENTITY     ",
      2294, "MQRC_UNKNOWN_REF_OBJECT      ",
      2295, "MQRC_CHANNEL_ACTIVATED       ",
      2296, "MQRC_CHANNEL_NOT_ACTIVATED   "
      };
    for (i=0;i<NO_OF_MQRCS;i++)
    {
       if(MQRC==MQRCTextArray??(i??).RC)
       {
          strcpy(MQRCText,MQRCTextArray??(i??).RCText);
          break;
       }
    } /* endfor */
 } /* End ConvertMQRC() */
