/*********************************************************************/
/*                                                                   */
/*  Module Name : READIMSG                                           */
/*                                                                   */
/*  Author      : Neil Johnston, June 1996                           */
/*                                                                   */
/*  Statement:    Licensed Materials - Property of IBM               */
/*                                                                   */
/*                MA1C SupportPac                                    */
/*                (c) Copyright IBM Corp. 1996.                      */
/*                                                                   */
/*                All rights reserved.                               */
/*                                                                   */
/*                U.S. Government Users Restricted Rights - use,     */
/*                duplication or disclosure restricted by GSA        */
/*                ADP Schedule Contract with IBM Corp.               */
/*                                                                   */
/*  Environment : MVS BATCH; C/370                                   */
/*                                                                   */
/*  Description : READIMSG formats and displays reply messages from  */
/*                IMS while filtering out the ll-zz's and any        */
/*                unnecessary newline charaters. The main() section  */
/*                of this program simply reads all the messages from */
/*                the specified queue one by one, displays the       */
/*                MQMD information and MQIIH information (if it      */
/*                is found) and passes a pointer to the remainder    */
/*                of the message to doImsPrint() which does the      */
/*                formatting.                                        */
/*                                                                   */
/*  Input       : 1. Queue manager to read messages from             */
/*                2. Queue to read messages from                     */
/*                                                                   */
/*  Output      : Output is written to STDOUT using printf with      */
/*                the ll-zz values for each line prefixed            */
/*                                                                   */
/*  Usage Notes : In order to prevent the ll-zz values for each line */
/*                appearing, comment out the indicated lines in the  */
/*                doImsPrint() function.                             */
/*                                                                   */
/*********************************************************************/

/* Defines for booleans **********************************************/

#define FALSE 0
#define TRUE  1

/* Include header files **********************************************/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <cmqc.h>

/* Function Prototypes ***********************************************/

void  usageError( char* programName );
void  errorMessage( char* msgStr, MQLONG CC, MQLONG RC );
void  printMD( MQMD* MDin);
void  printIIH( MQIIH* IIHin);
void  doImsPrint( char* start, char *end );

/* Main **************************************************************/

int main( int argc, char** argv )
{

   /* MQI variables */

   MQHCONN HConn      = MQHC_DEF_HCONN;
   MQMD    MsgDesc    = { MQMD_DEFAULT };
   MQOD    ObjDesc    = { MQOD_DEFAULT };
   MQHOBJ  HObj       = MQHO_UNUSABLE_HOBJ;
   MQGMO   GetMsgOpts = { MQGMO_DEFAULT };
   MQLONG  CompCode   = 0L;
   MQLONG  Reason     = 0L;

   /* Parameter variables */

   MQCHAR48 qMgr;
   MQCHAR48 qName;
   MQCHAR*  msgBuf = NULL;
   MQCHAR*  msgBufPtr = NULL;
   MQLONG   dataLength = 0L;
   MQLONG   outDataLength = 0L;

   /* Program variables */

   int      i = 0;
   int      moreMessages = TRUE;
   int      messageCount = 0;



   /* Set the variables to blanks */

   memset( qMgr,  ' ', MQ_Q_MGR_NAME_LENGTH );
   memset( qName, ' ', MQ_Q_NAME_LENGTH );

   /* Move parameters into corresponding variables */

   switch( argc )
   {
      case  3 : strncpy( qName, argv??( 2 ??), MQ_Q_NAME_LENGTH );
                strncpy( qMgr,  argv??( 1 ??), MQ_Q_MGR_NAME_LENGTH );
                break;
      default : usageError( argv??( 0 ??) );
                return 8;
   }

   /* Connect to Queue Manager (MQCONN) */

   MQCONN( qMgr,
           &HConn,
           &CompCode,
           &Reason     );

   /*  If connect failed then display error message and exit */

   if( MQCC_OK != CompCode )
   {
      errorMessage( "MQCONN", CompCode, Reason );
      return Reason;
   }

   /* Set queue name and open it */

   printf( "MQCONN SUCCESSFUL\nNow connected to: %s\n", qMgr );
   printf( "Opening queue: %s\n", qName );

   strncpy( ObjDesc.ObjectName, qName, MQ_Q_NAME_LENGTH );

   /* Open queue for browsing */

   MQOPEN( HConn,
           &ObjDesc,
           MQOO_BROWSE+MQOO_INPUT_EXCLUSIVE,
           &HObj,
           &CompCode,
           &Reason );

   if( MQCC_OK != CompCode )
   {
      errorMessage( "MQOPEN", CompCode, Reason );
      return Reason;
   }

   /* Loop until no more messages on queue */

   while( moreMessages )
   {

       /* Find out size of next message */

       GetMsgOpts.Options = MQGMO_BROWSE_FIRST +
                            MQGMO_ACCEPT_TRUNCATED_MSG;

       /* Clear ID's to pick up first msg on queue */

       for (i = 0 ; i < 24 ; i++)
           MsgDesc.MsgId??( i ??) = 0;

       for (i = 0 ; i < 24 ; i++)
           MsgDesc.CorrelId??( i ??) = 0;

       MQGET( HConn,
              HObj,
              &MsgDesc,
              &GetMsgOpts,
              0,           /* Buffer Len */
              NULL,        /* Buffer */
              &dataLength,
              &CompCode,
              &Reason );

       if( MQRC_NO_MSG_AVAILABLE == Reason )
       {
            break;  /* Fast escape from main loop */
       }

       /* We will accept truncated messages */

       if( MQCC_OK != CompCode &&
           MQRC_TRUNCATED_MSG_ACCEPTED != Reason )
       {
          errorMessage( "MQGET1", CompCode, Reason );
          return Reason;
       }

       /* Now allocate enough buffer space for pre-browsed msg */

       msgBuf = malloc( dataLength );

       if( ! msgBuf )
       {
          errorMessage( "Memory Alloction Error", 999, 999 );
          return Reason;
       }

       /* MsgId will still be set from browse so we'll get the
          message we want                                       */

       GetMsgOpts.Options = MQGMO_NONE;
       MQGET( HConn,
              HObj,
              &MsgDesc,
              &GetMsgOpts,
              dataLength,     /* Buffer Len */
              msgBuf,         /* Buffer */
              &outDataLength, /* Should become same as dataLength */
              &CompCode,
              &Reason );

       if( MQCC_OK != CompCode &&
           MQRC_NO_MSG_AVAILABLE != Reason )
       {
          errorMessage( "MQGET2", CompCode, Reason );
          return Reason;
       }

       if( MQRC_NO_MSG_AVAILABLE == Reason )
       {
            moreMessages = FALSE;
       }

       messageCount++;

       /* OK, lets have some useful output */

       printf( "--------------------------------------------------\n");
       printf( "Message  #%ld:\n", messageCount );
       printf( "Message Datalength is: %ld\n", dataLength);

       printMD( &MsgDesc );

       /* Check for MQIIH structure and display it. Warn user if
          structure not present but still format message         */

       if( 0 == strncmp( msgBuf, MQIIH_STRUC_ID,
                         sizeof( MQIIH_STRUC_ID ) ) )
       {
           printIIH( (MQIIH*) msgBuf );

           /* Skip MQIIH part of message, format and print the rest */

           msgBufPtr = ( msgBuf + sizeof( MQIIH ) );
           doImsPrint( msgBufPtr,
                    ( msgBufPtr + ( dataLength - sizeof( MQIIH ) ) ) );
       }
       else
       {
           printf("*** Warning: No MQIIH struct identifier in data\n");
           msgBufPtr = ( msgBuf );

           /* Format and print the message */

           doImsPrint( msgBufPtr,
                     ( msgBufPtr + dataLength ) );
       }

       /* Free up buffer memory */

       free( msgBuf );

       /* loop back until no more messages */
   }

   MQCLOSE( HConn,
            &HObj,
            MQCO_NONE,  /* No close options */
            &CompCode,
            &Reason );

   if( MQCC_OK != CompCode )
   {
      errorMessage( "MQCLOSE", CompCode, Reason );
      return Reason;
   }

   MQDISC( &HConn,
           &CompCode,
           &Reason    );

   if( MQCC_OK != CompCode )
   {
      errorMessage( "MQDISC", CompCode, Reason );
      return Reason;
   }
   else
   {
      printf( "MQDISC SUCCESSFUL\n" );
      return 0;
   }

}  /* end main */

/*********************************************************************/
/*                                                                   */
/* doImsPrint: This function formats and prints the IMS reply. It    */
/*             receives two arguments, a pointer to the start of the */
/*             message and a pointer to the end. It assumes it will  */
/*             always find at least one sequence of:                 */
/*                 ll-zz-message-newline                             */
/*                                                                   */
/*********************************************************************/

void doImsPrint( char *start, char *end )
{
    short int ll, zz;
    char* tempPtr;

   /* This routine assumes we have at least one 'll-zz-Message-0x15' */

   while( start < end )
   {
       /* I had to do a raw memcpy as I had trouble assigning values
          to a short int with the C/370 compiler                     */

       memcpy(&ll, start, 2);
       memcpy(&zz, start+2, 2);

       /* Display ll-zz. If you do not want the ll-zz values to be
          displayed then comment out the next two lines of code     */

       printf("%04X.%04X:", ll, zz );
       start += 4;

       /* IF we have 0x15('s), skip them */

       while (*start == 0x15)
       {
           start++;                 /* Skip 0x15 */
       }

       /* Remember start of string, go look for end, mark it with
          NULL and print this line                                   */

       tempPtr = start;
       while( (*start != 0x15) && (start < end) )
       {
           start++;
       }
       *start = 0;
       start++;

       printf("%s\n", tempPtr);

       /* Skip any terminating 0x15's and exit if we run out of
          buffer, otherwise loop repeatedly                     */

       while( (*start == 0x15) && (start < end) )
       {
           start++;
       }
   }
   return;
}

/* Display error message *********************************************/

void errorMessage( char* msgStr, MQLONG CC, MQLONG RC )
{
   printf( "************************************************\n" );
   printf( "* %s\n", msgStr );
   printf( "* COMPLETION CODE : %09ld\n", CC );
   printf( "* REASON CODE     : %09ld\n", RC );
   printf( "************************************************\n" );
}


/* Display usage error message ***************************************/

void usageError( char* programName )
{
   printf( "**************************************************\n" );
   printf( "PARAMETERS FOR %s\n", programName );
   printf( "     QMGR        - QUEUE MANGER\n" );
   printf( "     QNAME       - QUEUE NAME\n" );
   printf( "**************************************************\n" );
}

/* Print message descriptor ******************************************/

void printMD(MQMD *MDin)
{
   int i;
   printf("\n-----------------------------------------------------\n");
   printf("\n\nMessage Descriptor Structure \n");
   printf("\n  StrucId        : '%.4s'", MDin->StrucId);
   printf("\n  Version        : %d", MDin->Version);
   printf("\n  Report         : %d", MDin->Report);
   printf("\n  MsgType        : %d", MDin->MsgType);
   printf("\n  Expiry         : %d", MDin->Expiry);
   printf("\n  Feedback       : %d", MDin->Feedback);
   printf("\n  Encoding       : %d", MDin->Encoding);
   printf("\n  CodedCharSetId : %d", MDin->CodedCharSetId);
   printf("\n  Format         : '%.8s'", MDin->Format);
   printf("\n  Priority       : %d", MDin->Priority);
   printf("\n  Persistence    : %d", MDin->Persistence);

   printf("\n  MsgId          : 0X'");
   for (i = 0 ; i < 24 ; i++)
     printf("%02X",MDin->MsgId??( i ??) );
   printf("'");

   printf("\n  CorrelId       : 0X'");
   for (i = 0 ; i < 24 ; i++)
      printf("%02X",MDin->CorrelId??( i ??) );
   printf("'");

   printf("\n  BackoutCount   : %d", MDin->BackoutCount);
   printf("\n  ReplyToQ       : '%.48s'", MDin->ReplyToQ);
   printf("\n  ReplyToQMgr    : '%.48s'", MDin->ReplyToQMgr);
   printf("\n  UserIdentifier : '%.12s'", MDin->UserIdentifier);

   printf("\n  Account.Token  : 0X'");
   for (i = 0 ; i < 24 ; i++)
      printf("%02X",MDin->AccountingToken??( i ??) );
   printf("'");

   printf("\n  ApplIdentData  : '%.32s'", MDin->ApplIdentityData);
   printf("\n  PutApplType    : '%d'", MDin->PutApplType);
   printf("\n  PutApplName    : '%.28s'", MDin->PutApplName);
   printf("\n  PutDate        : '%.8s'", MDin->PutDate);
   printf("\n  PutTime        : '%.8s'", MDin->PutTime);
   printf("\n  ApplOriginData : '%.4s'\n", MDin->ApplOriginData);
   printf("\n-----------------------------------------------------\n");
}

/* Print IIH structure ***********************************************/

void printIIH(MQIIH *IIHin)
{
    int i;
    printf("\nIMS Bridge header Structure \n");
    printf("\n  StrucId           : '%.4s'", IIHin->StrucId);
    printf("\n  Version           : %d", IIHin->Version);
    printf("\n  StrucLength       : %d", IIHin->StrucLength);
    printf("\n  Encoding          : %d", IIHin->Encoding);
    printf("\n  CodedCharSetId    : %d", IIHin->CodedCharSetId);
    printf("\n  Format            : '%.8s'", IIHin->Format);
    printf("\n  Flags             : 0x%x", IIHin->Flags);
    printf("\n  LTermOverride     : '%.8s'", IIHin->LTermOverride);
    printf("\n  MFSMapName;       : '%.8s'", IIHin->MFSMapName);
    printf("\n  ReplyToFormat     : '%.8s'", IIHin->ReplyToFormat);
    printf("\n  Authenticator     : 0X'");
      for (i = 0 ; i < 8 ; i++)
        printf("%02X",IIHin->Authenticator??( i ??) );
      printf("'");
    printf("\n  TranInstanceId    : 0X'");
      for (i = 0 ; i < 16 ; i++)
        printf("%02X",IIHin->TranInstanceId??( i ??) );
      printf("'");
    printf("\n  TranState         : '%c'", IIHin->TranState);
    printf("\n  CommitMode        : '%c'", IIHin->CommitMode       );
    printf("\n  SecurityScope     : '%c'", IIHin->SecurityScope    );
    printf("\n-----------------------------------------------------\n");
    printf("\n\n");
}

/* End ***************************************************************/
